var calculator = (function(){
    var canvasMargin = 10;

    var height = 0;
    var skullHeight = 0;
    var shoulderWidth = 0;
    var pelvicWidth = 0;

    var calculations = {
        upperSides: 0,
        lowerSides: 0,
        side: 0,
        bottomHeight: 0,
        angles: {
            upperTop: 0,
            upperBottom: 0,
            lowerTop: 0,
            lowerBottom: 0
        }
    }

    var canvasElement = document.querySelector("#myCanvas");

    let dpi = window.devicePixelRatio;
    let style_height = +getComputedStyle(canvasElement).getPropertyValue("height").slice(0, -2);
    let style_width = +getComputedStyle(canvasElement).getPropertyValue("width").slice(0, -2);
    canvasElement.setAttribute('height', style_height * dpi);
    canvasElement.setAttribute('width', style_width * dpi);

    var canvas = {
        width: canvasElement.width,
        height: canvasElement.height,
        center: {
            x: canvasElement.width/2,
            y: canvasElement.height/2
        }
    }
    var coffinDrawingDims = {
        height: 0,
        shoulderWidth: 0,
        pelvicWidth: 0,
        topHeight: 0,
        bottomHeight: 0,
        upperSidesLength: 0,
        lowerSidesLength: 0
    }
    //(original height / original width) x new width = new height

    
    $('#height').change(function(){
        height = Number(this.value);
        calculator.update();
    });
    $('#skull-height').change(function(){
        skullHeight = Number(this.value);
        calculator.update();
    });
    $('#shoulder-width').change(function(){
        shoulderWidth = Number(this.value);
        calculator.update();
    });
    $('#pelvis-width').change(function(){
        pelvicWidth = Number(this.value);
        calculator.update();
    });

    var updateDrawing = function(){
        clearDrawing();

        var ratio = (canvas.height-(canvasMargin*2))/height;
        coffinDrawingDims.height = height*ratio;
        coffinDrawingDims.shoulderWidth = shoulderWidth*ratio;
        coffinDrawingDims.pelvicWidth = pelvicWidth*ratio;
        coffinDrawingDims.topHeight = skullHeight*ratio;
        coffinDrawingDims.bottomHeight = calculations.bottomHeight*ratio;
        coffinDrawingDims.upperSidesLength = calculations.upperSides*ratio;
        coffinDrawingDims.lowerSidesLength = calculations.lowerSides*ratio;

        var topCenter = [canvas.center.x, canvas.center.y-coffinDrawingDims.height/2];
        var bottomCenter = [canvas.center.x, topCenter[1]+coffinDrawingDims.height];
        var left = [canvas.center.x-coffinDrawingDims.shoulderWidth/2, topCenter[1]+coffinDrawingDims.topHeight];
        var right = [canvas.center.x+coffinDrawingDims.shoulderWidth/2, left[1]];

        var topLeft = [topCenter[0]-coffinDrawingDims.pelvicWidth/2, topCenter[1]];
        var topRight = [topCenter[0]+coffinDrawingDims.pelvicWidth/2, topCenter[1]];
        var bottomLeft = [bottomCenter[0]-coffinDrawingDims.pelvicWidth/2, bottomCenter[1]];
        var bottomRight = [bottomCenter[0]+coffinDrawingDims.pelvicWidth/2, bottomCenter[1]];

        console.debug(coffinDrawingDims, topCenter, bottomCenter, left, right, topLeft, topRight, bottomLeft, bottomRight);
        var context = canvasElement.getContext("2d");
        context.font = "30px Arial";

        // Cross Hairs
        context.setLineDash([5,3]);
        context.beginPath();
        context.moveTo(topCenter[0], topCenter[1]);
        context.lineTo(bottomCenter[0], bottomCenter[1]);
        context.closePath();
        context.strokeStyle = '#000';
        context.stroke();

        context.beginPath();
        context.moveTo(left[0], left[1]);
        context.lineTo(right[0], right[1]);
        context.closePath();
        context.strokeStyle = '#000';
        context.stroke();

        // Coffin
        context.beginPath();
        context.setLineDash([]);
        context.moveTo(topLeft[0], topLeft[1]);
        context.lineTo(topRight[0], topRight[1]);
        context.lineTo(right[0], right[1]);
        context.lineTo(bottomRight[0], bottomRight[1]);
        context.lineTo(bottomLeft[0], bottomLeft[1]);
        context.lineTo(left[0], left[1]);
        context.lineTo(topLeft[0], topLeft[1]);
        context.closePath();
        context.strokeStyle = '#000';
        context.stroke();

        //Meassurements
        context.fillText(height, topCenter[0]+canvasMargin, left[1]-coffinDrawingDims.topHeight/2+30);
        context.fillText(shoulderWidth, topCenter[0]+coffinDrawingDims.shoulderWidth/2/2, left[1]+30+canvasMargin);
        context.fillText(calculations.upperSides, topRight[0], topRight[1]);
        context.fillText(calculations.lowerSides, bottomRight[0], bottomRight[1]);
    }
    var clearDrawing = function(){
        var context = canvasElement.getContext('2d');
        context.clearRect(0, 0, canvasElement.width, canvasElement.height);
    }

    var angleC = function(lA, lB, lC){
        var a = Math.pow(lA, 2) + Math.pow(lB, 2) - Math.pow(lC, 2);
        var b = 2*lA*lB;
        return Math.acos(a/b)
    }

    return {
        update: function(){
            if(height > 0 && shoulderWidth > 0 && pelvicWidth > 0 && skullHeight > 0){
                calculations.side = (shoulderWidth-pelvicWidth)/2;
                calculations.bottomHeight = height-skullHeight;

                //Find the angled sides lengths
                calculations.upperSides = Math.sqrt(Math.pow(skullHeight,2)+Math.pow(calculations.side,2));
                calculations.lowerSides = Math.sqrt(Math.pow(calculations.bottomHeight,2)+Math.pow(calculations.side,2));
                calculations.angles.upperTop = angleC(skullHeight, calculations.upperSides, calculations.side);
                calculations.angles.upperBottom = angleC(calculations.upperSides, calculations.side, skullHeight);

                updateDrawing();
            } else {
                calculations.lowerSides = 0;
                calculations.upperSides = 0;

                clearDrawing();
            }
            console.debug({
                height: height,
                skullHeight: skullHeight,
                shoulderWidth: shoulderWidth,
                pelvicWidth: pelvicWidth,
                calculations: calculations
            });
        }
    }
})();
